function [r, c] = nxcorrfind( I, T );
%
% [r, c] = nxcorrfind( I, T ) - find maximum normalized cross correlation
%                               peak for targetin image
%
%   I       -   search image
%   T       -   target pattern
%
%   r,c     -   row/column coords of peak
%


% build pyramids for I and T
maxlevel = floor(log(min(size(T))) / log(2)) - 2;
pT = buildpyramid( T, maxlevel );
pI = buildpyramid( I, maxlevel );

% search on top level of pyramid
level = maxlevel;
[nrI, ncI] = size( pI{level} );
[nrT, ncT] = size( pT{level} );
f = nxcorr( pI{level}, pT{level}, 1, nrI-nrT+1, 1, ncI-ncT+1 );
[r, c] = find( f == max(f(:)) );

for level = maxlevel-1:-1:1
    % move coordinates from leve+1 to level
    r = (r-1) * 2 + 1;
    c = (c-1) * 2 + 1;
    
    % search in level in region of known maximum
    f = nxcorr( pI{level}, pT{level}, r-1, r+1, c-1, c+1 );
    [mxr, mxc] = find( f == max(f(:)) );
    
    % transfer coordinates from f to image
    r = mxr - 2 + r;
    c = mxc - 2 + c;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   f = nxcorr( I, T, r1, r2, c1, c2 ) - compute
%                   normalized cross correlation for T in region of I
%
%   I               -   search image
%   T               -   target pattern
%   r1,r2,c1,c2     -   search region
%
%   f               - normalized cross correlation
%

function f = nxcorr( I, T, r1, r2, c1, c2 )
    
    T = T / sqrt(sum(T(:).^2));
    I2 = I .^2;
    
    nr = r2 - r1 + 1;
    nc = c2 - c1 + 1;
    
    [nrT, ncT] = size(T);
    
    f = zeros( nr, nc );
    
    for i = r1:r2
        for j = c1:c2
            innerprod = sum(sum( T .* I( i:i+nrT-1, j:j+ncT-1 ) ));
            roimag = sqrt(sum(sum(I2(i:i+nrT-1, j:j+ncT-1))));
            f( i-r1+1, j-c1+1 ) = innerprod / ( roimag + eps );
        end
    end
    




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   pyramid = buildpyramid( I, maxlevel )
%
%       - build a pyramid from image I
%
%   I           -   image at base level of pyramid
%   maxlevel    -   highest level of pyramid
%
%   pyramid     -   cell array of pyramid layers
%                   (1 is bottom maxlevel is top)
%

function pyramid = buildpyramid( I, maxlevel )

[nr, nc] = size(I);
nr = floor(nr/2) * 2;
nc = floor(nc/2) * 2;
I = I(1:nr,1:nc);

pyramid(1) = {I};

for level = 2:maxlevel
    I = 0.25 * ( I(1:2:end,1:2:end) + I(2:2:end,1:2:end) + ...
                    I(1:2:end,2:2:end) + I(2:2:end,2:2:end) );
    pyramid(level) = {I};
    [nr, nc] = size(I);
    nr = floor(nr/2) * 2;
    nc = floor(nc/2) * 2;
    I = I(1:nr,1:nc);
end
